#!/usr/bin/perl -w
# blockCoverage.pl -- given a depth file from samtools, tab-delimited with columns
#	scaffold-name, position, #reads,
# and a fasta file, compute the average coverage for each n kB block
use strict;

{
    my $usage = "Usage: blockCoverage.pl fasta blockSizeKb depthFile > output";
    die $usage unless @ARGV==3;
    my ($fastaFile, $blockKb, $depthFile) = @ARGV;
    die $usage unless $blockKb =~ m/^\d+$/ && $blockKb > 0;
    my $blockSize = $blockKb * 1000;

    my %seqs = ();
    my $lastname = undef;
    open(FASTA, "<", $fastaFile) || die "Cannot read $fastaFile";
    while(<FASTA>) {
	chomp;
	if (m/^>(.*)$/) {
	    my $name = $1;
	    die "Duplicate sequences: $name" if exists $seqs{$name};
	    $seqs{$name} = "";
	    $lastname = $name;
	} else {
	    die "No header in $fastaFile" if !defined $lastname;
	    my $seq = $_;
	    $seq =~ s/\s//g; # remove whitespace
	    $seqs{$lastname} .= $seq;
	}
    }
    close(FASTA) || die "Error reading from $fastaFile";
    print STDERR "Read " . scalar(keys %seqs) . " sequences\n";

    # note blocks is using 0-based; not sure which samtools depth reports
    my %blocks = (); # scaffold => list, indexed by position/blockSize => [beg, end, fractionGC, totCoverage]
    while (my ($name,$sequence) = each %seqs) {
	my $len = length($sequence);
	$blocks{$name} = [];
	for (my $at = 0; $at + $blockSize <= $len; $at += $blockSize) {
	    my $subseq = substr($sequence, $at, $blockSize);
	    my @chars = split //, $subseq;
	    my %nChar = ();
	    map { $nChar{$_}++ } @chars;
	    my $nGC = $nChar{"G"} + $nChar{"C"};
	    my $nAT = $nChar{"A"} + $nChar{"T"};
	    my $fractionGC = ($nGC+$nAT > 0) ? $nGC/($nGC+$nAT) : "";
	    my $index = int($at/$blockSize);
	    $blocks{$name}[$index] = [ $at+1, $at+$blockSize, $fractionGC, 0];
	}
    }

    open(DEPTH, "<", $depthFile) || die "Error reading $depthFile";
    while(<DEPTH>) {
	chomp;
	my ($scaffold, $pos, $nReads) = split /\t/, $_;
	die "Invalid line $_" unless $pos =~ m/^\d+$/ && $nReads =~ m/^\d+$/;
	die "Unknown scaffold $scaffold" unless exists $blocks{$scaffold};
	die "Past end of scaffold $scaffold at pos $pos" if $pos > length($seqs{$scaffold});
	my $pos0 = $pos-1;
	print STDERR "Negative pos0!\n" if $pos0 < 0;
	next if $pos0 < 0;
	my $index = int($pos0/$blockSize);
	$blocks{$scaffold}[$index][3] += $nReads if exists $blocks{$scaffold}[$index];
    }
    close(DEPTH);

    print join("\t", qw{scaffoldId begin end GC depth})."\n";
    foreach my $name (sort keys %seqs) {
	my $blocks = $blocks{$name};
	foreach my $block (@{ $blocks{$name} }) {
	    my ($beg, $end, $GC, $tot) = @$block;
	    print join("\t", $name, $beg, $end, $GC, $tot/($end-$beg+1))."\n";
	}
    }
}
