#!/usr/bin/perl -w

# Create symbolic links to fastq files based on other/lane, which has
# a list of directories that contain fastq.gz files or individual
# fastq.gz files. The links are placed in subdirectories named BarSeq

my $debug = 0;
# these just print statements if $debug is set
sub mymkdir($);
sub mysymlink($$);

use strict;

{
    my $usage = "Usage: link_fastqs.pl source_directory1 ... source_directoryN\n";
    # optional debug mode with -debug -- list what would be done, but do not create any directories or symbolic links
    if (@ARGV > 0 && $ARGV[0] eq "-debug") {
	shift @ARGV;
	$debug = 1;
    }
    die $usage unless @ARGV > 0;
    my @source_dirs = @ARGV;
    foreach my $dir (@source_dirs) {
	die "No such directory: $dir\n$usage" unless -d $dir;
    }

    my $lanesfile = "other/lanes";
    die "Cannot find $lanesfile -- are you in the right directory?" if ! -e $lanesfile;

    if (-e "fastq") {
	die "Please rename or remove the fastq/ directory before running this script\n"
    }

    my @worklist = (); # list of $pathname, $newname
    open(LANES, "<", $lanesfile) || die "Cannot read $lanesfile";
    while(<LANES>) {
	chomp;
	next unless $_; # empty line
	next if m/^#/; # comment lines
	my @F = split /\t/, $_;
	next if @F == 0;
	die "Do not expect more than two fields in $lanesfile: $_" if @F > 2;
	my ($newname, $fastq) = @F;

	$fastq = $newname if !defined $fastq || $fastq eq "";

	die "Illegal output name $fastq" unless $fastq =~ m/\S/;

	# Look for the input file or directory
	my $pathname = undef;
	foreach my $dir (@source_dirs) {
	    my $try_pathname = "$dir/$fastq";
	    if (-e $try_pathname) {
		$pathname = $try_pathname;
		last;
	    }
	}
	die "Cannot find $fastq in any of the specified directories" if ! defined $pathname;

	# Verify that it is a directory that contains .fastq.gz files or it is a fastq.gz file
	if (-d $pathname) {
	    my @gzfiles = `find $pathname -name '*.fastq.gz' -print`;
	    die "No *.fastq.gz files in $pathname/\n" unless scalar(@gzfiles) > 0;
	} else {
	    die "Not a .fastq.gz file and not a directory: $pathname"
		unless $pathname =~ m/[.]fastq[.]gz$/;
	}
	push @worklist, [ $pathname, $newname ];
    }
    close(LANES) || die "Error reading $lanesfile";
    print STDERR "Read " . scalar(@worklist) . " items from $lanesfile\n";

    my $nDirMade++;
    my $nFilesLinked = 0;
    mymkdir("fastq") if ! -d "fastq";
    foreach my $row (@worklist) {
	my ($pathname,$newname) = @$row;
	my $newpath = "fastq/$newname";

	if (-d $pathname) {
	    # create pathname, create subdirectories as needed, and symlink all the .fastq.gz files"
	    mymkdir($newpath);
	    $nDirMade++;
	    my @gzfiles = `(cd $pathname; find . -name '*.fastq.gz' -print)`;
	    die "No *.fastq.gz files in $pathname/\n" unless scalar(@gzfiles) > 0;
	    foreach my $file (@gzfiles) {
		$file =~ s/\n$//; # remove new lines
		$file =~ s!^[.]/!!; # remove unnecessary ./ prefix
		if ($file =~ m!/[.]! || $file =~ m/^[.]/ || $file =~ m/^dana-/ || $file =~ m/^jayashree-/) {
		    print STDERR "Skipping $pathname/$file\n" if $debug;
		    next;
		}
		die $file if ! -e "$pathname/$file";
		my $parent = $file; $parent =~ s!/.*$!!;
		if ($parent ne $file && ! -d "$newpath/$parent") {
		    mymkdir("$newpath/$parent");
		    $nDirMade++;
		}
		mysymlink("$pathname/$file", "$newpath/$file");
		$nFilesLinked++;
	    }
	} else {
	    # symlink the file
	    mysymlink($pathname, $newpath);
	    $nFilesLinked++;
	}
    }
    print STDERR "Populated fastq/ with $nDirMade directories and $nFilesLinked symbolic links for *.fastq.gz files\n" unless $debug;
}

sub mymkdir($) {
    my ($dir) = @_;
    return if -d $dir;
    if ($debug) {
	print STDERR "Would mkdir $dir\n";
    } else {
	mkdir($dir) || die "mkdir $dir: $!";
    }
}

sub mysymlink($$) {
    my ($old,$new) = @_;
    if ($debug) {
	print STDERR "Would symlink $old $new\n";
    } else {
	symlink($old,$new) || die "symlink $old $new: $!";
    }
}
