#!/usr/bin/Rscript
# The orgs data structure includes for five organisms
#
# Keio is Escherichia coli K-12 (Keio parent strain)
# MR1 is Shewanella oneidensis MR-1
# SB2B is Shewanella amazonensis SB2B
# Phaeo is Phaeobacter inhibens DSM 17395 (formerly P. gallaeciensis, also referred to as the BS107 strain)
# psRCH2 is Pseudomonas stutzeri RCH2
#
# For each organism, orgs$name includes all the components of the fit data structure from FEBA_Fit(), as well as:
# genes -- describes the genes
# exps -- experiments
# pool10 -- the pool as defined by TnSeq 
# all - barseq counts for all barcodes in the pool for each sample (all)
# mogenes -- MicrobesOnline genes, includes TIGRfam roles
# gffmo -- mapping of MicrobesOnline ids to the ids in the genes table, if necessary
# auxo -- based on TIGR roles, genes that are expected to be involved in biosynthesis of amino acids or nucleotides
#
# The fit data structure includes:
# q -- experiment quality metrics (q$u=TRUE means the experiment passed quality metrics)
# g -- the genes with fitness data
# lrn -- per-gene fitness values (also see values for the 1st and 2nd half of each gene in lrn1, lrn2)
# t -- per-gene significance scores

source("feba/lib/FEBA.R");

LoadRBarSeq = function() {
	LoadFitnessData();
	LoadRBarSeqExtras();
}

LoadFitnessData = function() {
	orgs <<- list(); # <<- means save to parent environment
	for(org in words("Keio MR1 SB2B Phaeo psRCH2")) {
	    cat("Loading",org,"\n");

	    readOrgName = function(name) read.delim(paste("html/",org,"/",name,sep=""),check.names=FALSE, as.is=T);

	    # remove words after the experiment name from the tables as we have the metadata separately
	    readOrgName2 = function(name) {
	        d = readOrgName(name);
		names(d) = sub(" .*$", "", names(d));
		d;
	    }

	    orgs[[org]] <<- list(genes = readOrgName("genes"),
	    		         exps = readOrgName("expsUsed"),
				 q = readOrgName("fit_quality.tab"),
				 pool = readOrgName("pool"),
				 lrn = readOrgName2("fit_logratios.tab"),
				 lr = readOrgName2("fit_logratios_unnormalized.tab"),
				 t = readOrgName2("fit_t.tab"));
	    orgs[[org]]$g <<- orgs[[org]]$lrn$locusId;
	}
}

# merge together a bunch of tot2 files for an organism for quantitative analysis of TnSeq
LoadTnSeqTest = function(fileList, t0names, minT0=4) {
	tnseq = lapply(fileList, function(name)
			read.delim(paste("other_results/",name,".tot2",sep=""),
				as.is=T, header=F,
				col.names=words("scaffold pos strand locusId f n")));
	# each count column is named separately and then they are combined
	for(n in names(tnseq)) names(tnseq[[n]])[6] = n;
	tnseq_n = tnseq[[1]];
	for(i in 2:length(tnseq)) tnseq_n = merge(tnseq_n, tnseq[[i]], all=TRUE);
	for (i in names(tnseq)) tnseq_n[[i]] = na0(tnseq_n[[i]]);
	t0tot = rowSums(tnseq_n[,t0names,drop=F]);
	tnseq_n$u = !is.na(tnseq_n$f) & tnseq_n$f >= 0.1 & tnseq_n$f <= 0.9 & t0tot >= minT0;
	return(tnseq_n);
}

TnSeqTestGenesUsed12 = function(tnseq, t0names, minT0GeneSide=15) {
	t0tot = rowSums(tnseq[,t0names,drop=F]);
	n1 = aggregate(list(t0tot=t0tot[tnseq$u & tnseq$f < 0.5]), tnseq[tnseq$u & tnseq$f < 0.5,"locusId",drop=F], sum);
	n2 = aggregate(list(t0tot=t0tot[tnseq$u & tnseq$f >= 0.5]), tnseq[tnseq$u & tnseq$f >= 0.5,"locusId",drop=F], sum);
	return(intersect(n1$locusId[n1$t0tot >= minT0GeneSide], n2$locusId[n2$t0tot >= minT0GeneSide]));
}

LoadRBarSeqExtras = function(phase1only=FALSE) {

	# TnSeq data for some SB2B samples that we also have barseq for
	tnseq1 = LoadTnSeqTest(list(set2H13="Sample_KMW5_sb2b_gDNAplate1_F1_2H13", # Time0
			set2H14="Sample_KMW6_sb2b_gDNAplate1_F2_2H14", # Time0
			set2H15="Sample_KMW7_sb2b_gDNAplate1_F3_2H15", # D-Glucose
			set2H22="Sample_KMW8_sb2b_gDNAplate1_F10_2H22"), # D,L-Lactate
			t0names = words("set2H13 set2H14")); 
	tnseq1_fit = list();
	for (n in words("set2H15 set2H22"))
	    tnseq1_fit[[n]] = GeneFitness(orgs$SB2B$genes, tnseq1[,words("scaffold pos strand locusId f")],
		  		tnseq1[[n]], rowSums(tnseq1[,words("set2H13 set2H14")]),
				strainsUsed = tnseq1$u,
				genesUsed = orgs$SB2B$g,
				genesUsed12 = intersect(orgs$SB2B$g, TnSeqTestGenesUsed12(tnseq1, words("set2H13 set2H14"))));
	orgs$SB2B$tnseq_fit <<- tnseq1_fit;

	# TnSeq data for some psRCH2 samples that we also have barseq for
	tnseq2 = LoadTnSeqTest(list(set1H2="Sample_KMW2_psRCH2_gDNAplate3_a2_1H2", # Time0
				set1H3="Sample_KMW3_psRCH2_gDNAplate3_a3_1H31", # D-Glucose, set1H3
				set1H14="Sample_KMW4_psRCH2_gDNAplate3_b2_1H32"), # D,L-lactate, set1H14
				t0names = "set1H2");
	
	# At least 4 Time0 reads is the same threshold as for BarSeq
	tnseq2_fit = list();
	for (n in words("set1H3 set1H14"))
		tnseq2_fit[[n]] = GeneFitness(orgs$psRCH2$genes, tnseq2[,words("scaffold pos strand locusId f")],
		  		tnseq2[[n]], tnseq2$set1H2,
				strainsUsed = tnseq2$u,
				genesUsed = orgs$psRCH2$g,
				genesUsed12 = intersect(orgs$psRCH2$g, TnSeqTestGenesUsed12(tnseq2, "set1H2")));
	orgs$psRCH2$tnseq_fit <<- tnseq2_fit;

	# genome sequencing counts for regions of the SB2B chromosome, to verify chromosomal bias
	# The counts are averaged over regions of 10 kb, hence blocks10
	blockSpec = list("1H1"="S2", "1H3"="S3", "1H37"="S4", "1H40"="S6", "1H41"="S7", "1H42"="S5");
	blockFiles = list();
	for(n in names(blockSpec)) {
	    d1 = read.delim(paste("other_results/SB2B-",n,"_",blockSpec[[n]],
	                               "_L001_R1_001.blocks10",sep=""),
	       		     as.is=T);
	    d2 = read.delim(paste("other_results/SB2B-",n,"_",blockSpec[[n]],
	                           "_L001_R2_001.blocks10",sep=""),
			     as.is=T);
	    names(d1)[5] = "depth1";
	    d1$depth2 = d2$depth;
	    d1$depth = d1$depth1 + d2$depth;
	    blockFiles[[n]] = d1;
	}
	blocks10 = blockFiles[[1]][,1:4];
	for(n in names(blockFiles)) blocks10[[n]] = blockFiles[[n]]$depth;
	blocks10$u = rowSums(blocks10[,-(1:4)]) >= 6*50; # averaging at least 50 reads per block across 6 samples
	cat("Fraction of regions included in blocks10: ", mean(blocks10$u),"\n");
	blocks10 <<- blocks10;

	# Table of "old" per-gene fitness values for MR-1 using microarrays and two smaller pools, from Deutschbauer et al. 2014
	d = read.delim("other/mr1genes.mev",as.is=T,check.names=F);
	names(d) = sub(" .*", "", names(d));
	mrUarray <<- list(genes = d,
			exps = read.delim("other/mr1_experiments_grouped.tab",check.names=F,as.is=T));

 	# Nichols et al. fitness data for E. coli based on imaging colonies of Keio deletion strains
	keioG <<- read.delim("other/keio_growth.csv2",na.strings=c("NA","NT"));
	ecids <<- read.delim("other/ecids",header=F,as.is=T,col.names=c("ecocyc","name","BNum","eck","begin","end","strand","synonyms"));
	ecNichols = read.delim("other/coli_FinalData2.txt",as.is=T,check.names=F);
	ecNicholsEck = merge(ecids[,c("BNum","eck","name")],cbind(eck=sub("-.*","",ecNichols$Gene), ecNichols[,-1]));
	ecNicholsEck = merge(orgs$Keio$genes[,c("locusId","sysName")], ecNicholsEck, by.x="sysName", by.y="BNum");
	d = as.data.frame(table(ecNicholsEck$locusId));
	ecNicholsEck <<- ecNicholsEck[!ecNicholsEck$locusId %in% d$Var1[d$Freq>1],];
	nicholsCmps <<- read.delim("other/nicholsCmp",as.is=T); # mapping of corresponding experiments

	# auxotroph information
	for (n in names(orgs)) {
	    # mogenes includes "VIMSS" ids,
	    orgs[[n]]$mogenes <<- read.delim(paste("g/",n,"/mogenes",sep=""), as.is=TRUE, quote="");
	}
	for (n in names(orgs)) {
	    # MicrobesOnline ids for auxotrophs
	    mo_auxo = with(orgs[[n]]$mogenes, locusId[grepl("Amino acid biosynthesis|ribonucleotide biosynthesis", TIGRFunCat)]);
	    file = paste("g/",n,"/gffmo",sep="");
	    if (file.access(file,mode=4) == 0) { # readable
	        # This file is unnecessary if the gene identifiers in the fit data structure
		# or in genes$locusId are MicrobesOnline ids
	        cat("Reading mapping to MicrobesOnline ids for",n,"\n");
		orgs[[n]]$gffmo <<- read.delim(file,as.is=T);
		orgs[[n]]$auxo <<- with(orgs[[n]]$gffmo, locusId[VIMSS %in% mo_auxo]);
	    } else {
		orgs[[n]]$auxo <<- mo_auxo;
	    }
	}

	# Chimera testing with psRCH2 mini-pool (plate 17)
	# ignore S4, relatively few usable reads
	tnseq17 <<- sapply(words("S1 S2 S3 S5"),
		           function(n) read.delim(paste("other_results/psRCH2_minipool_",n,".pool",sep=""), as.is=T),
			   simplify=F);

	# Sanger mapping results of some of the same strains
	# includes plates other than 17 (for PlotChimeras)
	bestg17 <<- read.delim("other/RCH2_Sanger_2013Feb11.bestg",as.is=T);

	# Pools with barcodes seen just twice and no filtering (for PlotChimeras)
	lax <<- list(SB2B=read.delim("other_results/SB2B.pool.n2lax",as.is=T),
	    	     psRCH2=read.delim("other_results/psRCH2.pool.n2lax",as.is=T));

	# To illustrate effect on 1st-half/2nd-half plot of computing fitness in different ways
	SB2B_set1H10_gene_halves <<- read.delim("other_results/SB2B_set1H10_gene_halves.tab");
}

PlotChimeras = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=6, height=6, pointsize=10);
	oldpar = par(mfrow=c(2,2),mgp=c(2,1,0),mar=c(3,3,2,0)+0.5);

	d = merge(tnseq17[[1]], bestg17[substr(bestg17$read,0,2)=="17",], by="barcode");
	d = d[d$nTot >= 10 & d$scaffold != "pastEnd",];
	cat("Using ", nrow(d), "strains on plate 17\n");
	plot(d$n, d$nTot-d$n,
		  main="A. TnSeq of psRCH2 Mini-Pool",
		  xlab="Reads At Correct Location", ylab="Reads At Other Locations",
		  xlim=c(0,max(d$n)), ylim=c(0, max(d$n)/10));
	ratio = sum(d$nTot-d$n)/sum(d$n);
	cat("Ratio(nWrong/n) = ",ratio,"\n");
	cat("Ratio(n2nd/n) = ",sum(d$n2)/sum(d$n),"\n");
	abline(0,ratio,lty=2,col=8);
	abline(0,1,lty=2);

	set.seed(22072014);
	for (n in words("psRCH2 SB2B")) {
		d = lax[[n]];
		d = d[d$n >= 10 & d$scaffold != "pastEnd",];
		d = d[sample(1:nrow(d), 1000, replace=F),];
		plot(d$n, pmax(0.5,jitter(d$n2)),
			main=paste(ifelse(n=="psRCH2","B.","C."),"TnSeq of Full",n,"Pool"),
			xlab="Reads At Primary Location",
			ylab="Reads At Secondary Location",
			log="xy",
			col=ifelse(d$n >= 0.75 * d$nTot & d$n >= 8 * d$n2 & d$n >= 10, 1, 8));
		abline(0,1,lty=2);
	}

	par(oldpar);
	if(!is.null(file)) dev.off();
}

PlotPseudocounts = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=7.5, height=5, pointsize=10);
	oldpar = par(mfrow=c(2,3),mgp=c(2,1,0),mar=c(3,3,2,0)+0.5);

    # A. simulation
    set.seed(22072014);
    ngenes = 2000;
    nstrains = 20;
    nstrainstot = ngenes * nstrains;
    nreads = 20;
    nreadstot = nreads * nstrains * ngenes;
    nBad = 100;
    badHigh = sample(1:nstrainstot, nBad, replace=F);
    badZero = sample(setdiff(1:nstrainstot,badHigh), nBad, replace=F);

    genef = rnorm(ngenes)*1.5;
    straing = rep(1:ngenes, each=nstrains);
    strainf = rep(genef, each=nstrains);
    strainf[badHigh] = 6;
    strainf[badZero] = 0;

    strainN1 = rbinom(nstrainstot, nreadstot, 1/nstrainstot);
    strainN2 = rbinom(nstrainstot, nreadstot, 2**strainf/sum(2**strainf));

    geneFitness = AvgStrainFitness(strainN2, strainN1, straing);
    # or, with simpler pseudocounts
    geneFitness1 = AvgStrainFitness(strainN2, strainN1, straing, minGeneFactorNStrains=10000);

    plot(genef, geneFitness$fitNaive, main="A. 'Naive' Fitness From Total Counts",
    		xlab="Simulated Gene Fitness",
		ylab="Estimated Gene Fitness");
    eqline(lty=1);

    plot(genef, geneFitness$fit, main="B. Our Approach",
    		xlab="Simulated Gene Fitness",
		ylab="Estimated Gene Fitness");
    eqline(lty=1);

    plot(genef, geneFitness1$fit, main="C. Without Per-Gene Factor",
    		xlab="Simulated Gene Fitness",
		ylab="Estimated Gene Fitness");
    eqline(lty=1);

    # First/second-half quality plot for set1H10, doing the statistics in different ways
    # An older version of this was based on SB2Bparts <<- read.delim("workbook/rbarseq/SB2B_set1H10_part12.tab");
    counts = SB2B_set1H10_gene_halves;
    mednorm = function(x) x - median(x, na.rm=TRUE);
    counts$naive1 = mednorm(log2(1+counts$tot1) - log2(1+counts$tot1_0));
    counts$naive2 = mednorm(log2(1+counts$tot2) - log2(1+counts$tot2_0));
        
    plot(counts$naive1, counts$naive2, main="D. 'Naive' Fitness (SB2B)",
		xlab="First Half of Gene", ylab="Second Half of Gene");
    eqline(lty=1);

    plot(counts$lrn1, counts$lrn2, main="E. Our Approach (SB2B)",
		xlab="First Half of Gene", ylab="Second Half of Gene");
    eqline(lty=1);

    par(oldpar);
    if(!is.null(file)) dev.off();
}

sortedLines <- function(x, y, plot=FALSE, ...) {
	o <- order(x);
	if (plot) plot(x[o], y[o], type="l", ...) else lines(x[o], y[o], ...);
}

PlotQQ = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=7.5, height=5, pointsize=10);
	oldpar = par(mfrow=c(2,3),mgp=c(2,1,0),mar=c(3,3,2,0)+0.5);

	tot = 0;
	outliers = 0;
	for (i in 1:length(orgs)) {
	    n = names(orgs)[i];
	    pre = words("A B C D E")[i];
	    q = orgs[[n]]$q;
	    tvalues = orgs[[n]]$t;

	    Set = sub("(IT|H).*","",q$name);
	    ut0 = q$short=="Time0" & q$gMed >= 50;
	    if (n=="psRCH2") ut0 = ut0 & Set %in% words("set10 set11 set12 set13"); # new PCRs only
	    if (n=="Phaeo") ut0 = ut0 & Set %in% words("set5"); # new PCRs nly
	    time0s = intersect(names(tvalues), q$name[ut0]);
	    cat("Time0s for",n,time0s,"\n");

	    protgenes = with(orgs[[n]], g %in% genes$locusId[genes$type==1]);
	    nSigPhe = sum(abs(tvalues[protgenes,q$name[q$u]]) > 4);
	    nSigGenes = sum(rowSums(abs(tvalues[protgenes,q$name[q$u]]) > 4) > 0);
	    t0Phe = sum(abs(tvalues[protgenes,time0s,drop=F]) > 4);
	    t0PheScaled = t0Phe * sum(q$u)/length(time0s);

	    cat("Phenotypes for ", n, " with |t|>4: ", nSigPhe, "ProtGenes", nSigGenes,
	        "ControlPhenotypes", t0Phe, "Scaled", t0PheScaled,
		"FDR_Phe", t0PheScaled/nSigPhe,
		"FDR_Genes", t0PheScaled/nSigGenes,
		"\n");

	    tvalues = tvalues[,time0s,drop=F];	
	    tot = tot + length(time0s)*nrow(tvalues);
	    outliers = outliers + sum(abs(tvalues)>5);

	    plot(c(-4.5,4.5), c(-1,1)*max(abs(unlist(tvalues))), col=NA,
               xlab="Standard normal (sorted)", ylab="t statistic (sorted)",
	       main=sprintf("%s. t(Time0) in %s",pre,n));
            apply(tvalues, 2, function(t_vec) with(qqnorm(t_vec,plot=F), sortedLines(x,y, col="darkgrey")));
            apply(tvalues, 2, function(t_vec) with(qqnorm(t_vec,plot=F), points(x,y, col=ifelse(abs(y)>5,1,NA))));
            eqline(lty=1,col="black");
	}
	cat("total comparisons: ",tot,"outliers:",outliers,"\n");
	cat("expected:", pnorm(-5)*2*tot,"\n");
        if(!is.null(file)) dev.off();
}

MakeNicholsCmpData = function() {
	d = cbind(ecNicholsEck[,1:4], ecNicholsEck[,nicholsCmps$nname]);
	names(d)[-(1:4)] = nicholsCmps$name;
	d = d[rowSums(is.na(d[,-(1:4)])) == 0,];
	d2 = data.frame(locusId=orgs$Keio$g);
	for(i in 1:nrow(nicholsCmps)) d2[[ nicholsCmps$name[i] ]] = with(orgs$Keio,
		rowMeans(lrn[, q$name[q$short == nicholsCmps$short[i]]]) );
	ncData = merge(d,d2, suffixes=c(".n",""), by="locusId"); # 3375 rows
	return(ncData);
}

NicholsCmpPlot = function(condx, condy, Slow = -5, Shigh = -2, ...) {
    ncData = MakeNicholsCmpData();
    condxN = paste(condx,".n",sep="");
    condyN = paste(condy,".n",sep="");
    code = ifelse(ncData[[condxN]] < Slow & ncData[[condyN]] > Shigh, 1,
    	          ifelse(ncData[[condxN]] > Shigh & ncData[[condyN]] < Slow, 2, 3));
    plot(ncData[[condx]], ncData[[condy]], xlab=condx, ylab=condy,
		col=c(2,4,8)[code], pch=c(2,5,20)[code]);
    points(ncData[[condx]][code!=3], ncData[[condy]][code!=3], col=c(2,4,8)[code[code!=3]], pch=c(2,5,20)[code[code!=3]]);
    # highlight outliers?
    outX = code==1 & !(ncData[[condx]] < -1 & ncData[[condx]] < ncData[[condy]]-1);
    if(sum(outX) > 0) text(ncData[[condx]][outX], ncData[[condy]][outX], ncData$name[outX], col=2, pos=3); # pos=3 means above
    outY = code==2 & !(ncData[[condy]] < -1 & ncData[[condy]] < ncData[[condx]]-1);
    if(sum(outY) > 0) text(ncData[[condx]][outY], ncData[[condy]][outY], ncData$name[outY], col=4, pos=1); # pos=1 means below
    abline(0,1);
    rbXonly = code==3 & ncData[[condx]] < -2 & ncData[[condy]] > -1;
    if(sum(rbXonly) > 0) cat("Random barseq only for",condx,sort(ncData$name[rbXonly]),"\n");
    rbYonly = code==3 & ncData[[condy]] < -2 & ncData[[condx]] > -1;
    if(sum(rbYonly) > 0) cat("Random barseq only for",condy,sort(ncData$name[rbYonly]),"\n");
}

NicholsCmpPlots = function(file=NULL) {
    if(!is.null(file)) pdf(file, width=7.5,height=7.5,pointsize=10);
    oldpar = par(mfrow=c(2,2),mgp=c(2,1,0),mar=c(3,3,2,0)+0.5);
    NicholsCmpPlot("Acetate","Glucosamine");
    NicholsCmpPlot("Glucose","Glycerol");
    NicholsCmpPlot("Maltose","NAG");
    NicholsCmpPlot("NAG","Succinate");
    par(oldpar);
    if(!is.null(file)) dev.off();
}

UarrayCmpPlot = function(file=NULL) {
	if(!is.null(file)) pdf(file,width=3,height=3,pointsize=10);
	oldpar = par(mgp=c(2,1,0),mar=c(4,3,2,1)+0.5);
	d = merge(cbind(locusId=mrUarray$genes$gene, old=mrUarray$genes$"36"),
	          cbind(locusId=orgs$MR1$g, new=orgs$MR1$lrn$set2H45));
	plot(d$old, d$new, xlab="Fitness from microarray & defined pools", ylab="Fitness from Random BarSeq", main="Technology Comparison:\nFitness on L-lactate (MR1)");
	eqline(col=1,lty=1);
	text(-5, 3, sprintf("r = %.2f", cor(d$new,d$old)));
	par(oldpar);
	if(!is.null(file)) dev.off();
}

ChromosomalBiasCmpPlot = function(n, t0) {
    plot(blocks10$begin/1e3, mednorm(log2(blocks10[[n]]/blocks10[[t0]])),
         xlab="Position on Chromosome (kilobases)", ylab="Log2 Ratio",
	 ylim=c(-0.7,0.7),
	 main = sprintf("S. amazonensis SB2B, %s vs. %s", n, t0));
    nfull = paste("set",n,sep="");
    d = with(orgs$SB2B, merge(data.frame(locusId=g, normdiff=lr[[nfull]]-lrn[[nfull]]),
				data.frame(locusId=genes$locusId, mid=(genes$begin+genes$end)/2))); 
    sortedLines(d$mid/1e3, d$normdiff, col=3, lwd=2);
    legend("topleft", c("Observed","Estimated"), pch=c(1,NA), lty=c(NA,1), col=c(1,3), lwd=c(1,3), bty="n");
}

ChromosomalBiasCmpPlots = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=6, height=6, pointsize=10);
	oldpar = par(mfrow=c(2,2));
	ChromosomalBiasCmpPlot("1H3","1H1");
	ChromosomalBiasCmpPlot("1H40","1H37");
	ChromosomalBiasCmpPlot("1H41","1H37");
	ChromosomalBiasCmpPlot("1H42","1H37");
	par(oldpar);
	if(!is.null(file)) dev.off();
}

TnSeqCmpPlot = function(org, name, conddesc, cex=0.75) {
	d = with(orgs[[org]],
	  merge(data.frame(locusId=lrn$locusId, rbarseq=lrn[[name]], t=t[[name]]), tnseq_fit[[name]][,words("locusId fitnorm")]));
	plot(d$rbarseq, d$fitnorm,
			xlab="Random BarSeq", ylab="TnSeq", cex=cex,
			main = sprintf("Gene Fitness in %s (%s)", conddesc, org),
			col = ifelse(abs(d$t) > 4, "red", "darkgrey"));
	points(d$rbarseq, d$fitnorm, cex=cex, col=ifelse(abs(d$t) > 4, 2, NA));
	eqline(col=1, lty=1);
	hline(0, col=1, lty=1);
	vline(0, col=1, lty=1);
	legend("bottomright", "|t| > 4", col=2, pch=1);
}

TnSeqCmpPlots = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=7, height=7, pointsize=10);
	oldpar = par(mfrow=c(2,2));
	TnSeqCmpPlot("SB2B", "set2H15", "D-Glucose");
	TnSeqCmpPlot("SB2B", "set2H22", "D,L-Lactate");
	TnSeqCmpPlot("psRCH2", "set1H3", "D-Glucose");
	TnSeqCmpPlot("psRCH2", "set1H14", "D,L-Lactate");
	par(oldpar);
	if(!is.null(file)) dev.off();
}

MadReadsPlots = function(file=NULL) {
	if(!is.null(file)) pdf(file, width=6.75, height=4.5, pointsize=10);
	oldpar = par(mgp=c(2,1,0), mar=c(4,3,2,1)+0.5, mfrow=c(2,3));

	# First panel explains the median absolute difference between the two halves, using
	# SB2B set1H10  (D,L-Lactate, mad12 = 0.18)
	d = median(abs(SB2B_set1H10_gene_halves$lrn1 - SB2B_set1H10_gene_halves$lrn2),na.rm=T);
	plot(SB2B_set1H10_gene_halves$lrn1, SB2B_set1H10_gene_halves$lrn2,
		pch=20, cex=0.5,
		xlim=c(-2,2), ylim=c(-2,2),
		xlab = "Fitness, 1st Half of Gene",
		ylab = "Fitness, 2nd Half of Gene",
		main = "A. SB2B in D,L-Lactate" );
	abline(0,1,col=4); abline(d,1,lty=2,col=4); abline(-d,1,lty=2,col=4);
	text(2, -2, sprintf("m.a.d. = %.2f", d), adj=c(1,0)); # specify right bottom

	mains = list(SB2B="B. Shewanella amazonensis SB2B",
	             MR1="C. Shewanella oneidensis MR-1",
		     psRCH2="D. Pseudomonas stutzeri RCH2",
		     Keio="E. Escherichia coli K-12",
		     Phaeo="F. Phaeobacter inhibens BS107");
	ymax = max(sapply(orgs, function(x) with(x, max(q$mad12))));
	for (org in words("SB2B MR1 psRCH2 Keio Phaeo")) with(orgs[[org]], {
	    d = q[q$short != "Time0",];
	    plot(d$gMed, d$mad12,
	         main = mains[[org]],
	         xlab="Median Reads per Gene",
		 ylab="M.a.d.(1st half,2nd half)",
		 xlim = c(0, max(d$gMed)),
		 ylim = c(0, ymax),
	         col = ifelse(d$maxFit > 6, 2, 1));
	    if (org=="SB2B") {
	        d = d[d$name=="set1H10",];
		arrows(d$gMed, d$mad12 + 0.1, d$gMed, d$mad12, length=0.06);
	    }
	    hline(0.5);
	    vline(50);
	    legend("topright", "max(fit) > 6", col=2, pch=1);
	});
	par(oldpar);
	if(!is.null(file)) dev.off();
}

SavePlots = function(plots = words("PlotChimeras PlotPseudocounts PlotQQ NicholsCmpPlots UarrayCmpPlot ChromosomalBiasCmpPlots TnSeqCmpPlots MadReadsPlots")) {
	for (funcname in plots) {
	      filename = paste("plots/",funcname,".pdf",sep="");
	      cat("Saving ",filename,"\n");
	      do.call(funcname, list(file=filename));
	}
}

# Automatically make the image
if (!interactive()) {
   LoadRBarSeq();
   cat("rbarseq.R loaded data succesfully\n");
   dir.create("plots", showWarnings=FALSE);
   save.image("plots/rbarseq.image");
   SavePlots();
   quit();
}
